'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createTest = require('./create-test');

var _createTest2 = _interopRequireDefault(_createTest);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var reduxRecord = function reduxRecord(_ref) {
  var reducer = _ref.reducer;
  var _ref$includeReducer = _ref.includeReducer;
  var includeReducer = _ref$includeReducer === undefined ? false : _ref$includeReducer;
  var stateKey = _ref.stateKey;
  var actionSubset = _ref.actionSubset;
  var _ref$equality = _ref.equality;
  var equality = _ref$equality === undefined ? '(result, nextState) => result === nextState' : _ref$equality;
  var _ref$imports = _ref.imports;
  var imports = _ref$imports === undefined ? '' : _ref$imports;
  var _ref$testLib = _ref.testLib;
  var testLib = _ref$testLib === undefined ? 'tape' : _ref$testLib;
  var _ref$numTestsToSave = _ref.numTestsToSave;
  var numTestsToSave = _ref$numTestsToSave === undefined ? 5 : _ref$numTestsToSave;
  var _ref$includeShowTests = _ref.includeShowTestsButton;
  var includeShowTestsButton = _ref$includeShowTests === undefined ? false : _ref$includeShowTests;

  var actions = [];
  var recording = false;
  var showingTest = false;
  var testIndex = undefined;
  var prevTests = [];
  var listeners = [];
  var stringifiedReducer = '/* import reducer from YOUR_REDUCER_LOCATION_HERE */';
  if (includeReducer) {
    stringifiedReducer = 'var reducer = ' + reducer.toString();
  }

  var updateListeners = function updateListeners() {
    if (listeners.length) {
      (function () {
        var newTestState = {
          recording: recording,
          showingTest: showingTest,
          tests: prevTests,
          testIndex: testIndex
        };
        listeners.forEach(function (fn) {
          return fn(newTestState);
        });
      })();
    }
  };

  // we can skip type checking to see if equality arg is a string since string.toString() returns same string
  var equalityFunction = equality.toString();
  var startRecord = function startRecord() {
    recording = true;
    updateListeners();
  };
  var stopRecord = function stopRecord() {
    recording = false;
    showingTest = true;
    updateListeners();
  };

  var hideTest = function hideTest() {
    showingTest = false;
    updateListeners();
  };

  var showTest = function showTest() {
    showingTest = true;
    updateListeners();
  };

  var emptyActions = function emptyActions() {
    actions = [];
    updateListeners();
  };

  var updateTestIndex = function updateTestIndex(index) {
    testIndex = index;
    updateListeners();
  };

  var genTest = typeof testLib === 'function' ? testLib : _createTest2.default;
  if (typeof testLib !== 'string' && typeof testLib !== 'function') {
    throw new Error('testLib argument must be a string or a function');
  }
  if (typeof testLib === 'string' && !(0, _createTest.isTestLibrarySupported)(testLib)) {
    throw new Error('testLib argument does not contain a supported testing library. ' + 'Feel free to make a pr adding support or use a custom test generation function for testLib arg');
  }

  var createNewTest = function createNewTest() {
    var test = genTest({
      actions: actions,
      imports: imports,
      equalityFunction: equalityFunction,
      reducer: stringifiedReducer,
      testLib: testLib
    });
    if (prevTests.length === numTestsToSave) {
      prevTests = prevTests.slice(1, prevTests.length);
    }
    prevTests.push(test);
    testIndex = prevTests.length - 1;
    updateListeners();
    return test;
  };

  var middleware = function middleware(_ref2) {
    var getState = _ref2.getState;
    return function (next) {
      return function (action) {
        if (recording) {
          var prevState = stateKey ? getState()[stateKey] : getState();
          next(action);
          var nextState = stateKey ? getState()[stateKey] : getState();
          if (!actionSubset || actionSubset[action.type]) {
            actions.push({ action: action, prevState: prevState, nextState: nextState });
          }
        } else {
          next(action);
        }
      };
    };
  };

  var listen = function listen(fn) {
    listeners.push(fn);
    updateListeners();
    return function () {
      return listeners.splice(listeners.splice(listeners.indexOf(fn), 1));
    };
  };
  var props = {
    startRecord: startRecord,
    stopRecord: stopRecord,
    createNewTest: createNewTest,
    hideTest: hideTest,
    showTest: showTest,
    emptyActions: emptyActions,
    updateTestIndex: updateTestIndex,
    listen: listen,
    includeShowTestsButton: includeShowTestsButton
  };
  return { middleware: middleware, props: props };
};

exports.default = reduxRecord;